package Scorch.Items;

/*
  Class:  Item
  Author: Mikhail Kruk
  Description: the super class of all the items (shield, fuel etc)
*/

import java.util.*;
import java.awt.*;

import Scorch.ScorchApplet;

public abstract class Item
{
    private static String pkgName = "Scorch.Items.";

    public static final int Shield = 0, MediumShield = 1, HeavyShield = 2,
	Parachute = 3, Battery = 4, Tracer = 5, AutoDefense = 6, Fuel = 7;

    private static final String names[] = {
	"Shield", "Medium Shield", "Heavy Shield",
	"Parachute", "Battery", "Tracer", "Auto Defense"//, "Fuel"
    };

    protected long price; // price 0 means that this item can't be bought
    protected int quantity = 0;
    protected int type;

    protected Label lQuantity = null;

    protected ItemControl controlPanel = null;
    
    public String getName()
    {
	return names[type];
    }
    
    public int getType()
    {
	return type;
    }

    public long getPrice()
    {
	return price;
    }

    public int getQuantity()
    {
	return quantity;
    }

    public void incQuantity(int inc)
    {
	setQuantity( quantity+inc );
    }

    public void decQuantity()
    {
	setQuantity( quantity-1 );
    }

    public void setQuantity(int q)
    {
	quantity = q;
	if( lQuantity == null ) 
	    lQuantity = new Label(quantity+"", Label.CENTER);
	else
	    lQuantity.setText(quantity+"");

	if( controlPanel != null )
	    {
		controlPanel.setEnabled(quantity > 0);
		controlPanel.validate();
	    }
    }

    public Label getQuantityLabel()
    {
	if( lQuantity == null )    // if label not created yet create it now
	    setQuantity(quantity);
	
	return lQuantity;
    }

    public static Vector loadItems()
    {
	Vector result = new Vector();
	
	try
	    {
		for(int i = 0; i < names.length; i++)
		    {
			String name = trimSpaces(pkgName, names[i]);

			result.addElement(Class.forName(name).newInstance());
		    }
	    }
	catch(Exception e)
	    {
		System.out.println("Failed to load weapons: "+e);
	    }
	
	return result;
    }

    // each item and weapons must create control panel which
    // will be then displayed in the Invetory box, save it in
    // the controlPanel field and return it
    abstract public ItemControl getControlPanel(ScorchApplet owner);

    public static String trimSpaces(String pName, String fullName)
    {
	String name = pName;

	StringTokenizer st = new StringTokenizer(fullName, " ");
	while( st.hasMoreTokens() ) name += st.nextToken();
	
	return name;
    }
}
